invoice	date	task	duration hours	duration (unit)
{% for task in tasks -%}
{{ task.invoice }}	{{ task.date.date() }}	{{ task.task }}	{{ task.worked_time }}	{{ task.worked_time_friendly }}
{% endfor -%}
