{% for day in data -%}
{{ day.date.strftime('%A')  }} {{ day.date.strftime('%m-%d')  }}: {{ day.worked_time }}
{% for task in day.tasks %} {{ task.task }} - {{ task.task_time }}
{% endfor %}
{% endfor -%}
